#include <string>
#include <iostream>
#include <fstream>

using std::string;

class person {
public:
    explicit person(string);

    person(string name, string surname)
        :m_name(name), m_surname(surname)
    { ; }

    void set_name(string name){ m_name = name; }
    void set_surname(string surname){ m_surname = surname; }

    const string & get_name() const { return m_name; }
    const string & get_surname() const { return m_surname; }

private:
    string m_name;
    string m_surname;
};

// operator vystupu - musi vracet referenci na vystupni proud,
// abychom mohli delat cout << franta << " " << lenka
// pricemz uzavorkovani je zleva doprava
// nemuze byt metodou, protoze metody o n argumentech se prekladaji tak ze
// jsou brany jako funkce o n+1 argumentech pricemz onim 0. argumentem je
// samotna instance objektu
// tento operator by tedy mohl byt metodou, ale pouze nad typem std::ostream
std::ostream & operator<<(std::ostream & output, const person & osoba){
    output << osoba.get_name() << " " << osoba.get_surname();
    return output;
}

// operator vstupu - v podstate stejny jako operator vystupu,
// rozdil je jen v nekonstantni referenci na druhy argument (logicky, nemohli
// bychom jinak zmenit cilovy objekt)
std::istream & operator>>(std::istream & input, person & osoba){
    string tmp;

    input >> tmp;
    osoba.set_name(tmp);

    input >> tmp;
    osoba.set_surname(tmp);

    return input;
}

int main(){
    person novakovi[] = {
        person("Franta", "Novak"),
        person("Tonda", "Novak"),
        person("Lenka", "Novakova"),
        person("Alena", "Novakova")
    };

    // v hlavice fstream se nachazi datovy typ fstream, souborovy proud
    std::fstream vystup;
    // podobne jako v C je potreba soubor nejprve otevrit - prvnim argumentem
    // je cesta k souboru, druhym pak bitove pole znacici rezim otevreni
    vystup.open("rodina_novakova.txt", std::ios_base::out | std::ios_base::trunc);

    for (int i = 0; i < 4; i++){
        vystup << novakovi[i] << std::endl;
    }

    // soubor pochopitelne zavreme - zajisi vyliti bufferu
    // pozn: u textovych souboru ukoncovanych znakem noveho radku pomoci std::endl
    // toto neni doopravdy potreba, nebot std::endl nad danym proudem vola flush(),
    // toto je ale zmineno jen pro zajimavost, vzdy soubor zavirejte!
    vystup.close();

    return 0;
}
